#include "AntitrendFilter.h"


AntitrendFilter::AntitrendFilter(int nLeads, int avm, int sampleRate)
{
	int i = 0;
	this->nInputLeads = nLeads;
	this->sampleRate = sampleRate;
	this->avm = avm;
	trendOrder = 1; //  
	trendFreq = 0.01f; //     0.0
	trendFreqOutOfRange = 2.0f; //     rangeLimit 
	rangeLimit = 3.0f; //    ,        trendFreqOutOfRange 
	trendEnvFiltLength = 1.0f; // , 

	//  BesselHighPass
	Dsp::Spec spec;
	memset(&spec, 0, sizeof(Dsp::Spec));
	spec.cutoffFreq = trendFreq;
	spec.centerFreq = trendFreq;
	spec.sampleRate = sampleRate;
	spec.order = trendOrder;
	//    
	trendFilter = new Dsp::BesselHighPass<50, 1>[nInputLeads];
	//  
	for (i = 0; i < nInputLeads; i++) 
		trendFilter[i].Setup(spec);

	//     
	int envCoreSamples = (int) (trendEnvFiltLength * sampleRate);
	float *envCore = new float [envCoreSamples];
	//      1.0
	for (i = 0; i < envCoreSamples; i++)
	{
		envCore[i] = 1.0f;
	}
	//  
	envFilter = 0;
	envFilter = new FIRFilter [nInputLeads];
	//    
	for (i = 0; i < nInputLeads; i++)
	{
		envFilter[i].setCore(envCore, envCoreSamples);
	}
	delete envCore;
}


AntitrendFilter::~AntitrendFilter(void)
{
	delete [] trendFilter;
	delete [] envFilter;
}


void AntitrendFilter::processChunk(float **inpLeads, int inpLen)
{
	int i = 0, chan = 0, pId = 0;
	float **prePrInput = new float * [nInputLeads];
	//    
	float rangeLimitScaled = (rangeLimit * 1000000.0f / avm);
	bool outOfRange = false;
	float envMax = 0.0f;
	for (chan = 0; chan < nInputLeads; chan++)
	{
		//   
		prePrInput[chan] = new float [inpLen];
		memcpy(prePrInput[chan], inpLeads[chan], sizeof(float) * inpLen);
		//  
		envFilter[chan].process(prePrInput[chan], inpLen);
		//       
		// 
		for (i = 0; i < inpLen; i++)
			envMax = max(envMax, fabs(prePrInput[chan][i]));

		memcpy(prePrInput[chan], inpLeads[chan], sizeof(float) * inpLen);
		//   
		//(      DspFilter )
		for (i = 0; i < inpLen; i++)
			prePrInput[chan][i] = -prePrInput[chan][i];
	}
	//    
	float trendAlpha = fabs(min(1.0f, envMax / rangeLimitScaled));
	//   
	float curTrendFreq = ((1.0f - trendAlpha) * trendFreq + 
		trendAlpha * trendFreqOutOfRange);

	//  
	Dsp::Spec spec;
	memset(&spec, 0, sizeof(Dsp::Spec));
	spec.cutoffFreq = curTrendFreq;
	spec.sampleRate = sampleRate;
	spec.order = trendOrder;
	spec.centerFreq = spec.cutoffFreq;

	// 
	for (chan = 0; chan < nInputLeads; chan++)
	{
		trendFilter[chan].Setup(spec);
		trendFilter[chan].Process(inpLen, prePrInput[chan]);
		memcpy(inpLeads[chan], prePrInput[chan], sizeof(float) * inpLen);
	}

	for (chan = 0; chan < nInputLeads; chan++)
	{
		delete prePrInput[chan];
	}
	delete prePrInput;
}

void AntitrendFilter::process(float **data, int len)
{
	int chunkLen = sampleRate;
	int i = 0;
	float **chunkPtrs = new float *[nInputLeads];
	int chan = 0, curChunkLen = 0;
	for (i = 0; i < len; i += chunkLen)
	{
		for (chan = 0; chan < nInputLeads; chan++)
		{
			chunkPtrs[chan] = data[chan] + i;
		}
		curChunkLen = min(chunkLen, len - i);
		if (curChunkLen <= 0) break;
		processChunk(chunkPtrs, curChunkLen);
	}
	delete chunkPtrs;
}

void AntitrendFilter::reset()
{
	int chan = 0;
	for (chan = 0; chan < nInputLeads; chan++)
	{
		trendFilter[chan].Reset();
	}
}

void AntitrendFilter::setAVM(float val)
{
	avm = val;
}

void AntitrendFilter::setSampleRate(int sampleRate)
{
	this->sampleRate = sampleRate;
}

