#pragma once
#include <memory.h>
#include <stdlib.h>

class FIRFilter
{
private:
	float *core;
	int nCoreSamples;
	float *buffer;
	float mul;
	float amplify;
	
	float step(float inp)
	{
		int i = 0;
		for (i = 0; i < nCoreSamples - 1; i++)
			buffer[i] = buffer[i + 1];
		buffer[nCoreSamples - 1] = inp;
		double sum = 0.0f;
		for (i = 0; i < nCoreSamples; i++)
			sum += buffer[i] * core[i];
		return sum * mul;
	}
public:
	FIRFilter(void) : 
		  core(NULL),
		  buffer(NULL),
		  nCoreSamples(0),
		  mul(1.0f),
		  amplify(1.0f)
	{
	}
	~FIRFilter(void)
	{
		if (core)
			delete core;
		if (buffer)
			delete buffer;
	}
	void setCore(float *newCore, int nNewCoreSamples)
	{
		if (core)
			delete core;
		if (buffer)
			delete buffer;
		nCoreSamples = nNewCoreSamples;
		core = new float [nCoreSamples];
		buffer = new float [nCoreSamples];
		memset(buffer, 0, sizeof(float) * nCoreSamples);
		float sum = 0.0f;
		int i = 0;
		for (i = 0; i < nNewCoreSamples; i++)
			sum += newCore[i];
		mul = 1.0f / sum;
		/*for (i = 0; i < nCoreSamples; i++)
		{
			core[i] = newCore[i];
		}*/
		memcpy(core, newCore, nCoreSamples * sizeof(float));
	}
	void process(float *dst, int nSamples)
	{
		if (!dst || !nSamples) return;
		if (!core) return;
		int i = 0;
		if (amplify == 1.0f)
		{
			for (i = 0; i < nSamples; i++)
			{
				dst[i] = step(dst[i]);
			}
		}
		else
			for (i = 0; i < nSamples; i++)
			{
				dst[i] = amplify * step(dst[i]);
			}
	}
	void reset()
	{
		memset(buffer, 0, sizeof(float) * nCoreSamples);
	}
	float getAmplify() const { return amplify; }
	void setAmplify(float val) { amplify = val; }
	int getCoreLength() const { return nCoreSamples; }
};
